; Expand the perimeter to reflect the given new cluster point
; A helper function for eden_growth
; Assumes that the point is indeed part of the cluster whose perimeter is given
pro expand_perimeter,cluster,dim,perimeter,point
    ; This array depends on the geometry of the lattice
    adjacent_points = find_adjacent(point, dim)

    for i = 0, n_elements(adjacent_points)-1 do begin ; for every point adjacent to the cluster's new aquisition
       cur_adj_pt = adjacent_points[i]
       ; If this point is neither part of the cluster nor part of the old
       ; perimeter ...
       if cluster[cur_adj_pt] EQ 0 then begin
         ; ... we have a new perimeter point.
         ; Add point to the perimeter points list
         perimeter[0]++
         perimeter[perimeter[0]] = cur_adj_pt
         ; Mark the point as occupied
         cluster[cur_adj_pt] = 1
       endif
    endfor
    ; Note that since we are not removing any cluster points here, we need not
    ; worry about removing perimeter points.
end

; pro eden_growth,sites_bi,perim_bi,final_size

; Use the eden growth process to "expand" the given cluster to the desired size.
function eden_growth,cluster,final_size
    current_size = n_bi_points(cluster.sites)
    points_to_add = final_size - current_size
    dim = final_size*2l + 1l

    ; Quick error-checking
    if points_to_add LT 0l then message,'final_size - current_size must be positive, not ' + string(points_to_add)

    ; ~~~ Build the cluster and perimeter lists in a form appropiate for
    ; processing
    ; List of the cluster's points. The first element contains the number of
    ; points currently stored. Sized to be able to contain the maximum
    ; anticipated cluster sized plus the length element.
    cluster_sites = lonarr(final_size + 1, /NOZERO)
    cluster_sites[1:current_size] = bi2uni(cluster.sites,dim,final_size)
    cluster_sites[0] = current_size
    ; List of the cluster's perimeter points. Sized to be able to contain
    ; the maximum possible perimeter length (final_size*2 + 2) plus the length
    ; element.
    current_perim_size = n_bi_points(cluster.perim)
    cluster_perimeter = lonarr((final_size*2 + 2) + 1, /NOZERO)
    cluster_perimeter[1:current_perim_size] = bi2uni(cluster.perim,dim,final_size)
    cluster_perimeter[0] = current_perim_size
    ; Build the array of occupied sites using the info in cluster_sites and
    ; cluster_perimeter
    cluster_occupied = bytarr(dim, dim)
    cluster_occupied[ cluster_sites[1:cluster_sites[0]] ] = 1
    cluster_occupied[ cluster_perimeter[1:cluster_perimeter[0]] ] = 1

    ; ~~~ Add the remaining points via an Eden growth process
    while points_to_add GT 0 do begin
       ; Randomly select a perimeter point
       perim_pt_index = long(randomu(seed)*(cluster_perimeter[0])) + 1
       perim_pt = cluster_perimeter[perim_pt_index]
       ; Add that point to the cluster
       cluster_sites[0]++
       cluster_sites[cluster_sites[0]] = perim_pt
       ; Remove that point from the perimeter list (and do so in a way that
       ; keeps the list compact)
       cluster_perimeter[perim_pt_index] = cluster_perimeter[cluster_perimeter[0]]
       cluster_perimeter[0]--
       ; Expand the perimeter to reflect the new point
       expand_perimeter, cluster_occupied, dim, cluster_perimeter, perim_pt
       ; Deincriment point counter
       points_to_add--
    endwhile

    ; ~~~ Reformat sites and perimeter lists and send them back
    sites_bi = uni2bi(cluster_sites[1:cluster_sites[0]], cluster_occupied, final_size)
    perim_bi = uni2bi(cluster_perimeter[1:cluster_perimeter[0]], cluster_occupied, final_size)
    return, {sites:sites_bi, perim:perim_bi}
end
